`timescale 1 ns / 1 ps

module IFIDReg( 
	input wire LoadEnable,
	input wire CLK,
	input wire RESET,
	input wire [31:0] IF_Inst,
	input wire [31:0] IF_NextPC,
	output reg [31:0] ID_Inst,
	output reg [31:0] ID_NextPC
	);
	
	always @ ( posedge CLK ) begin
			if(RESET) begin
					ID_Inst <= 32'd0;
					ID_NextPC <= 32'd0;
			end else if ( LoadEnable) begin
					ID_Inst <= IF_Inst; // only update ID_Inst
					ID_NextPC <= IF_NextPC;   // only update ID_NextPC
			end	else begin
					ID_Inst <= ID_Inst; // Hold current value
					ID_NextPC <= ID_NextPC; // Hold current value
				end
		end
endmodule

module IDEXReg(
	input wire LoadEnable,
	input wire CLK,
	input wire RESET,
	/////// INPUT CONTROL ///////
	/* Control Signals generated for use in EX, */
	/* Control Signals generated for use in MEM, */
	/* Control Signals generated for use in WB, */	
	input wire ID_RegWrite,	
	input wire [4:0] ID_WriteReg, /* Destination Register of ID_Inst */
	input wire [31:0] ID_Inst, /* Instruction occupying the ID Stage */
	/////// INPUT DATA ///////
	input wire [31:0] ID_RS_OUT,
	input wire [31:0] ID_RT_OUT,
	/* Other ID DATA SIGNALS, */  
	/////// OUTPUT CONTROL ///////
	/* Control signals consumed in EX , */
	/* Control signals passed thru EX for use in MEM , */
	/* Control signals passed thru EX for use in WB , */	
	output reg EX_RegWrite,
	output reg [4:0] EX_WriteReg, /* Destination Register of EX_Inst */
	output reg [31:0] EX_Inst, /* Instruction occupying the EX Stage */
	/////// OUTPUT DATA ///////
	output reg [31:0] EX_RS_IN,
	output reg [31:0] EX_RT_IN  /* ADD COMMA!!! */
	/* Other EX DATA SIGNALS, */
	);
	
	always @ ( posedge CLK ) begin
			if(RESET) begin
					/* Control Signals consumed in EX = 0 */
					/* Control Signals passed thru EX = 0 */
					EX_RegWrite <= 1'b0;
					EX_RS_IN <= 32'd0;
					EX_RT_IN <= 32'd0;
					/* Other EX DATA SIGNALS = 0 */
					EX_Inst <= 32'd0;
					EX_WriteReg <= 5'b00000;
			end else if ( LoadEnable) begin
					/* Control Signals consumed in EX = Control Signals generated for use in EX */
					/* Control Signals passed thru EX for MEM = Control Signals generated for use in MEM */
					/* Control Signals passed thru EX for WB = Control Signals generated for use in WB */	
					EX_RegWrite <= ID_RegWrite;
					EX_RS_IN <= ID_RS_OUT;
					EX_RT_IN <= ID_RT_OUT;
					/* Other EX DATA SIGNALS = Other ID DATA SIGNALS */
					EX_Inst <= ID_Inst;
					EX_WriteReg <= ID_WriteReg; 
			end else begin
					/* Control Signals consumed in EX = Control Signals consumed in EX */
					/* Control Signals passed thru EX for MEM = Control Signals passed thru EX for MEM */
					/* Control Signals passed thru EX for WB = Control Signals passed thru EX for WB */	
					EX_RegWrite <= EX_RegWrite;
					EX_RS_IN <= EX_RS_IN;
					EX_RT_IN <= EX_RT_IN;
					/* Other EX DATA SIGNALS = Other EX DATA SIGNALS */
					EX_Inst <= EX_Inst;
					EX_WriteReg <= EX_WriteReg; 
				end
		end
endmodule

module EXMEMReg( 
	input wire LoadEnable,
	input wire CLK,
	input wire RESET,
	/////// INPUT CONTROL ///////
	/* Control Signals passed thru EX for MEM, */
	/* Control Signals passed thru EX for WB, */	
	input wire EX_RegWrite,
	input wire [4:0] EX_WriteReg, /* Destination Register of EX_Inst */
	input wire [31:0] EX_Inst, /* Instruction occupying the EX Stage */
	/////// INPUT DATA ///////
	input wire [31:0] EX_ALUOut,
	input wire [31:0] EX_CPUDataOut,
	/////// OUTPUT CONTROL ///////
	/* Control Signals consumed in MEM, */
	/* Control Signals passed thru MEM for WB, */		
	output reg MEM_RegWrite,
	output reg [4:0] MEM_WriteReg, /* Destination Register of MEM_Inst */
	output reg [31:0] MEM_Inst, /* Instruction occupying the MEM Stage */
	/////// OUTPUT DATA ///////
	output reg [31:0] MEM_ALUOut,
	output reg [31:0] MEM_CPUDataOut
	);
	
	always @ ( posedge CLK ) begin
			if(RESET) begin
					/* Control Signals consumed in MEM = 0 */
					/* Control Signals passed thru MEM = 0 */
					MEM_RegWrite <= 1'b0;
					MEM_WriteReg <= 5'b00000;
					MEM_Inst <= 32'd0;
					MEM_ALUOut <= 32'd0;
					MEM_CPUDataOut <= 32'd0;
			end else if ( LoadEnable) begin
					/* Control Signals consumed in MEM = Control Signals passed thru EX for MEM */
					/* Control Signals passed thru MEM for WB = Control Signals passed thru EX for WB */	
					MEM_RegWrite <= EX_RegWrite;
					MEM_WriteReg <= EX_WriteReg;
					MEM_Inst <= EX_Inst;
					MEM_ALUOut <= EX_ALUOut;
					MEM_CPUDataOut <= EX_CPUDataOut;
			end else begin
					/* Control Signals consumed in MEM = Control Signals consumed in MEM */
					/* Control Signals passed thru MEM for WB = Control Signals passed thru MEM for WB */	
					MEM_RegWrite <= MEM_RegWrite;
					MEM_WriteReg <= MEM_WriteReg;
					MEM_Inst <= MEM_Inst;
					MEM_ALUOut <= MEM_ALUOut;
					MEM_CPUDataOut <= MEM_CPUDataOut;
				end
		end
endmodule

module MEMWBReg( 
	input wire LoadEnable,
	input wire CLK,
	input wire RESET,
	/////// INPUT CONTROL ///////
	/* Control Signals passed thru MEM for WB, */
	input wire MEM_RegWrite,
	input wire [4:0] MEM_WriteReg, /* Destination Register of MEM_Inst */
	input wire [31:0] MEM_Inst,
	/////// INPUT DATA ///////
	input wire [31:0] MEM_ALUOut,
	input wire [31:0] MEM_CPUDataIn,
	/////// OUTPUT CONTROL ///////
	/* Control Signals consumed in WB */
	output reg WB_RegWrite,
	output reg [4:0] WB_WriteReg, /* Destination Register of WB_Inst */
	output reg [31:0] WB_Inst,
	/////// OUTPUT DATA ///////
	output reg [31:0] WB_ALUOut,
	output reg [31:0] WB_CPUDataIn
	);
	
	always @ ( posedge CLK ) begin
			if ( RESET ) begin
					/* Control Signals consumed in WB = 0 */
					WB_RegWrite <= 1'b0;
					WB_ALUOut <= 32'd0;
					WB_CPUDataIn <= 32'd0; 
					WB_Inst <= 32'd0;
					WB_WriteReg <= 5'b00000;
			end else if ( LoadEnable ) begin
					/* Control Signals consumed in WB = Control Signals passed thru MEM for WB */
					WB_RegWrite <= MEM_RegWrite;
					WB_ALUOut <= MEM_ALUOut;
					WB_CPUDataIn <= MEM_CPUDataIn;
					WB_Inst <= MEM_Inst;
					WB_WriteReg <= MEM_WriteReg;
			end else begin
					/* Control Signals consumed in WB = Control Signals consumed in WB */
					WB_RegWrite <= WB_RegWrite;
					WB_ALUOut <= WB_ALUOut;
					WB_CPUDataIn <= WB_CPUDataIn;
					WB_Inst <= WB_Inst;
					WB_WriteReg <= WB_WriteReg;
				end	
		end
endmodule